@@ -25,8 +25,7 @@
 		[dataFile release];
 		return nil;
 	}
-	// max file size is 2GB or 31bits. So unsigned int is ok
-	[dataFile setSize:[[fileAttributes objectForKey:NSFileSize] unsignedIntValue]];
+	[dataFile setSize:[[fileAttributes objectForKey:NSFileSize] unsignedLongLongValue]];
 	//	[dataFile setTimestamp:[fileAttributes objectForKey:NSFileModificationDate]];
 	return [dataFile autorelease];
 }
@@ -43,12 +42,12 @@
 
 // accessor methods
 
-- (void)setSize:(unsigned)newSize
+- (void)setSize:(unsigned long long)newSize
 {
 	size = newSize;
 }
 
-- (unsigned)size
+- (unsigned long long)size
 {
 	return size;
 }
