@@ -258,14 +258,13 @@ StatusDisplayer *statusDisplayerGlobal;
 	
 	if (mtpDevice)
 	{
-		PTPDeviceInfo deviceInfo;
-		if (ptp_getdeviceinfo(&params,&deviceInfo) == PTP_RC_OK)
+		if (ptp_getdeviceinfo(&params,&(params.deviceinfo)) == PTP_RC_OK)
 		{
-			deviceString = [NSString stringWithCString:deviceInfo.Model];
-			deviceIDString = [NSString stringWithCString:deviceInfo.SerialNumber];
+			deviceString = [NSString stringWithCString:params.deviceinfo.Model];
+			deviceIDString = [NSString stringWithCString:params.deviceinfo.SerialNumber];
 			
 			// the version string is of the form firmware_device
-			NSString *versionString = [NSString stringWithCString:deviceInfo.DeviceVersion];
+			NSString *versionString = [NSString stringWithCString:params.deviceinfo.DeviceVersion];
 			NSArray *versions = [versionString componentsSeparatedByString:@"_"];
 			if ([versions count] != 2)
 			{
@@ -329,6 +328,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		else
 		{
 			close_device(&ptp_usb, &params, interfaceNumber);
+			ptp_free_deviceinfo(&params.deviceinfo);
 		}
 		connected = NO;
 		[self setStatus:STATUS_DISCONNECTED];
@@ -514,20 +514,11 @@ StatusDisplayer *statusDisplayerGlobal;
 		}
 		int i = 0;
 		for (i = 0; i < params.handles.n; i++) {
-			if (ptp_getobjectinfo(&params,params.handles.Handler[i], &oi) == PTP_RC_OK)
+			if (ptp_getobjectinfo(&params, params.handles.Handler[i], &oi) == PTP_RC_OK)
 			{
+				/*
 				if (oi.Filename != NULL)
-					NSLog(@"handle: 0x%08.x, filename: %s", params.handles.Handler[i], oi.Filename);
-				
-				uint32_t* ohArray = NULL;
-				uint32_t arrayLen = 0;
-				if (ptp_getobjectreferences (&params, params.handles.Handler[i], &ohArray, &arrayLen) == PTP_RC_OK)
-				{
-					int j = 0;
-					for (j = 0; j<arrayLen; j++)
-						NSLog(@"object 0x%08.x references object 0x%08.x", params.handles.Handler[i], ohArray[j]);
-					free(ohArray);
-				}
+					NSLog(@"handle: 0x%08.x, filename: %s", params.handles.Handler[i], oi.Filename); */
 				
 				if (oi.ObjectFormat == PTP_OFC_Association || 
 						(oi.ObjectFormat != PTP_OFC_WAV && oi.ObjectFormat != PTP_OFC_MP3 && oi.ObjectFormat != PTP_OFC_WMA))
@@ -603,20 +594,18 @@ StatusDisplayer *statusDisplayerGlobal;
 					unicodevalue = NULL;
 				}
 				
-				/* I don't think this is supported yet.
 				char *timeString= NULL;
 				ret = ptp_getobjectpropvalue(&params, PTP_OPC_OriginalReleaseDate, params.handles.Handler[i], (void**)&timeString, PTP_DTC_STR);
 				if (ret == PTP_RC_OK && timeString != NULL)
 				{
 					NSString *nsTimeString = [NSString stringWithCString:timeString];
-					NSLog(@"Got time string %@", nsTimeString);
 					free(timeString);
-					// we're ignoring the timezone here...
-					// and the Creative devices don't send seconds like they should!
+					timeString = NULL;
+					// the Creative devices don't send seconds (but they do send tenths of a second!)
 					NSCalendarDate *date = [[NSCalendarDate alloc] initWithString:nsTimeString calendarFormat:@"%Y%m%dT%H%M"];
 					
 					[track setYear:[date yearOfCommonEra]];
-				}*/
+				}
 				
 				[track setItemID:params.handles.Handler[i]];
 				
@@ -680,7 +669,8 @@ StatusDisplayer *statusDisplayerGlobal;
 				break;
 		}
 		
-		if (send_file(&params, [[track fullPath] fileSystemRepresentation], format, progressPTP, &handle) == -1)
+		if (send_file(&params, [[track fullPath] fileSystemRepresentation], [[[track fullPath] lastPathComponent] fileSystemRepresentation],
+									format, progressPTP, &handle) != PTP_RC_OK)
 		{
 			NSString *error = @"Error uploading file";
 			NSLog(error);
@@ -732,6 +722,7 @@ StatusDisplayer *statusDisplayerGlobal;
 		int ret = ptp_deleteobject(&params, [track itemID],0);
 		if (ret != PTP_RC_OK)
 		{
+			ptp_perror(&params, ret);
 			NSString *error = @"error deleting file";
 			NSLog(error);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
@@ -760,14 +751,21 @@ StatusDisplayer *statusDisplayerGlobal;
 			NSLog(error);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
 		}
-		else
-		{
-			[self updateDiskSpace];
-			return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
-		}
 	}
 	else
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
+	{
+		int ret = ptp_deleteobject(&params, [fileID unsignedIntValue], 0);
+		if (ret != PTP_RC_OK)
+		{
+			ptp_perror(&params, ret);
+			NSString *error = @"error deleting file";
+			NSLog(error);
+			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+		}
+	}
+	
+	[self updateDiskSpace];
+	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
 
 - (NJBTransactionResult *)downloadTrack:(Track *)track
@@ -886,12 +884,11 @@ StatusDisplayer *statusDisplayerGlobal;
 	ret = ptp_setobjectpropvalue(&params, PTP_OPC_Duration, [track itemID], &durationMS, PTP_DTC_UINT32);
 	uint16_t trackNumber = [track trackNumber];
 	ret = ptp_setobjectpropvalue(&params, PTP_OPC_Track, [track itemID], &trackNumber, PTP_DTC_UINT16);
-	/* I don't think times are implemented yet.  Try this and you'll get garbage back!
-	NSString *nsTimeString = [NSString stringWithFormat:@"%.04d0101T0000.0", [track year]];
-	NSLog(@"Setting time string to %@", nsTimeString);
+
+	NSString *nsTimeString = [NSString stringWithFormat:@"%.04d0101T000000.0", [track year]];
 	const char *timeString = [nsTimeString cString];
-	ret = ptp_setobjectpropvalue(&params, PTP_OPC_OriginalReleaseDate, [track itemID], &timeString, PTP_DTC_STR);
-	*/
+	ret = ptp_setobjectpropvalue(&params, PTP_OPC_OriginalReleaseDate, [track itemID], (void*)timeString, PTP_DTC_STR);
+	
 	if (ret != PTP_RC_OK)
 		return NO;
 	else
@@ -1065,10 +1062,41 @@ StatusDisplayer *statusDisplayerGlobal;
 	}
 	else
 	{
-		uint32_t handle = 0;
+		uint16_t objectFormat = PTP_OFC_Undefined;
 		
-		if (send_file(&params, [[dataFile fullPath] fileSystemRepresentation], PTP_OFC_Undefined, progressPTP, &handle) == -1)
+		// a test to see how the player works with video & images
+		NSString *extension = [[[dataFile fullPath] pathExtension] lowercaseString];
+		if ([extension isEqualToString:@"wmv"])
+			objectFormat = PTP_OFC_WMV;
+		else if ([extension isEqualToString:@"avi"])
+			objectFormat = PTP_OFC_AVI;
+		else if ([extension isEqualToString:@"mpg"] || [extension isEqualToString:@"mpeg"])
+			objectFormat = PTP_OFC_MPEG;
+		else if ([extension isEqualToString:@"asf"])
+			objectFormat = PTP_OFC_ASF;
+		else if ([extension isEqualToString:@"qt"])
+			objectFormat = PTP_OFC_QT;
+		else if ([extension isEqualToString:@"jpg"] || [extension isEqualToString:@"jpeg"])
+			objectFormat = PTP_OFC_JFIF; // or should this be PTP_OFC_EXIF_JPEG?
+		else if ([extension isEqualToString:@"tif"] || [extension isEqualToString:@"tiff"])
+			objectFormat = PTP_OFC_TIFF;
+		else if ([extension isEqualToString:@"bmp"])
+			objectFormat = PTP_OFC_BMP;
+		else if ([extension isEqualToString:@"gif"])
+			objectFormat = PTP_OFC_GIF;
+		else if ([extension isEqualToString:@"pict"])
+			objectFormat = PTP_OFC_PICT;
+		else if ([extension isEqualToString:@"png"])
+			objectFormat = PTP_OFC_PNG;
+		
+		NSLog(@"uploading file with object format 0x%.02x", objectFormat);
+		
+		uint32_t handle = 0;
+		uint16_t ret = send_file(&params, [[dataFile fullPath] fileSystemRepresentation], [[[dataFile fullPath] lastPathComponent] fileSystemRepresentation],
+														 objectFormat, progressPTP, &handle);
+		if (ret != PTP_RC_OK)
 		{
+			/* we might get an (MTP error) object too large here */
 			NSString *error = @"Error uploading file";
 			NSLog(error);
 			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
@@ -1137,6 +1165,10 @@ StatusDisplayer *statusDisplayerGlobal;
 	else
 	{
 		// note Creative devices seem not to support this
+		
+		if (!ptp_property_issupported(&params, PTP_DPC_DateTime))
+			return nil;
+		
 		char *timeString = NULL;
 		if (ptp_getdevicepropvalue(&params, PTP_DPC_DateTime, (void **)&timeString, PTP_DTC_STR) != PTP_RC_OK)
 			return nil;
@@ -1164,14 +1196,26 @@ StatusDisplayer *statusDisplayerGlobal;
 
 - (NJBTransactionResult *)setOwnerString:(NSString *)owner
 {
-	if (mtpDevice)
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO] autorelease];
-	
-	if (NJB_Set_Owner_String (njb, [owner UTF8String]) == -1)
+	if (!mtpDevice)
 	{
-		NSString *error = [self njbErrorString];
-		NSLog(error);
-		return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+		if (NJB_Set_Owner_String (njb, [owner UTF8String]) == -1)
+		{
+			NSString *error = [self njbErrorString];
+			NSLog(error);
+			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+		}
+	}
+	else
+	{
+		unsigned short *unistring = [UnicodeWrapper UTF16FromString:owner];
+		uint16_t ret = ptp_setdevicepropvalue(&params, PTP_DPC_DeviceFriendlyName, unistring, PTP_DTC_UNISTR);
+		free(unistring);
+		if (ret != PTP_RC_OK)
+		{
+			NSString *error = [NSString stringWithFormat:@"Could not set device friendly name, return value 0x%04.x", ret];
+			NSLog(error);
+			return [[[NJBTransactionResult alloc] initWithSuccess:NO resultString:error] autorelease];
+		}
 	}
 	return [[[NJBTransactionResult alloc] initWithSuccess:YES] autorelease];
 }
@@ -1380,6 +1424,12 @@ StatusDisplayer *statusDisplayerGlobal;
 		
 		if (playlistState == NJB_PL_NEW)
 		{
+			/* note that we really want to create a new playlist not by using
+			SendObjectInfo but with the enhanced SendObjectPropList. Then we can send
+			filesize 0 (we still have to send object, but with no data). Sending filesize
+			0 with SendObjectInfo gives error PTP_RC_StoreFull. So hopefully we can persuade
+			Microsoft to let us use SendObjectPropList. */
+			
 			PTPObjectInfo newInfo;
 			memset(&newInfo, 0, sizeof(newInfo));
 			[playlist lock];
@@ -1672,7 +1722,7 @@ int progressPTP(u_int32_t sent, u_int32_t total)
 		uint8_t *value = NULL;
 		if (ptp_getdevicepropvalue(&params,PTP_DPC_BatteryLevel,(void**)&value,PTP_DTC_UINT8) != PTP_RC_OK)
 		{
-			NSLog(@"Could not get battery level property description");
+			NSLog(@"Could not get battery level property value");
 			return 0;
 		}
 				
