@@ -49,9 +49,9 @@ typedef enum _codecTypes {CODEC_UNDEFINED = -1, CODEC_MP3 = 0, CODEC_WMA = 1, CO
 - (NSString *)codecString;
 - (NSString *)njbCodec;
 - (void)setCodec:(codecTypes)newCodec;
-- (unsigned)filesize;
+- (unsigned long long)filesize;
 - (NSString *)filesizeString;
-- (void)setFilesize:(unsigned)newFilesize;
+- (void)setFilesize:(unsigned long long)newFilesize;
 - (void)setValuesToTrack:(Track *)track;
 
 - (NSComparisonResult)compareByLength:(Track *)other;
