@@ -118,7 +118,7 @@
     int offset = 0;
     
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
-    
+	
     NSMutableData * v2Tag;
     
     NSFileHandle * file = [NSFileHandle fileHandleForReadingAtPath:path];
@@ -126,22 +126,22 @@
     if (file == NULL)
     {
         NSLog(@"Can not open file :%s",[path cString]);
+				[pool release];
         return NO;
     }
 
     BOOL headerFound = NO;
     fileSize = [file seekToEndOfFile];
     
-    
     if (exhastiveSearch) bufferSize = 8192; // make storage buffer large if exhastively searching the file.
     
     if (fileSize < bufferSize) bufferSize = fileSize;
-    
+		
     for (position = 0; position < fileSize; position += bufferSize - 9 - 6) // header size = 9 , eHeader =6
     { // try to find a id3 v2 tag grabs file data and scans for header.
         
         [file seekToFileOffset:position];
-        v2Tag = (NSMutableData *) [file readDataOfLength: bufferSize];
+				v2Tag = (NSMutableData *) [file readDataOfLength: bufferSize];
         
         if ((start = [self scanForHeader:v2Tag]) >= 0)
         {
@@ -170,21 +170,21 @@
 				[self parseExtendedHeader:extendedHeader];
             }    
             // get the frameSet
-            if (start + 10 + tagLength > bufferSize) {
+           if (start + 10 + tagLength > bufferSize) {
 				[file seekToFileOffset:position+start+extendedHeaderLength+10]; 
                 v2Tag = (NSMutableData *) [file readDataOfLength: tagLength - extendedHeaderLength];
                 offset = extendedHeaderLength;
             } else {
                 offset = start + 10 + extendedHeaderLength;
             }
-            /*if ([self tagUnsynch]) 
+            if ([self tagUnsynch]) 
             {
                 v2Tag = [self desynchData:v2Tag offset:offset];
                 offset = 0;
-            }*/
+            }
             // create a processing object for the tag version
            
-             switch (majorVersion)
+            switch (majorVersion)
             {
                 case 0	: {
                                 frameSet = [[NSMutableDictionary alloc] initWithCapacity:10];
@@ -291,7 +291,7 @@
             minorVersion = Buffer[i+4];
             flag = Buffer[i+5];
             tagLength = [self readPackedLengthFrom: Buffer + i + 6];
-            // finds tag length from header
+            // finds tag length from header						
             return i;
         }
     }
@@ -1179,7 +1179,8 @@
                 NSString * yearString = [anObject getTextFromFrame];
                 NSArray *listItems = [yearString componentsSeparatedByString:@"-"];
                 yearString = [listItems objectAtIndex:0];
-                [listItems release];
+								// BUG: listItems must not be released here. Fixed by RAL 2006-01-07
+                ///////[listItems release];
                 return [yearString intValue];
             }
         }
