@@ -456,8 +456,7 @@ ptp_unpack_DPV (PTPParams *params, char* data, void** value, uint16_t datatype)
 		case PTP_DTC_STR:
 		{
 			uint8_t len;
-			char* charValue = (char *)(*value);
-			charValue=ptp_unpack_string(params,data,0,&len);
+			(char *)(*value)=ptp_unpack_string(params,data,0,&len);
 			return 2*len+1;
 			break;
 		}
@@ -846,8 +845,7 @@ ptp_pack_DPV (PTPParams *params, void* value, char** dpvptr, uint16_t datatype)
 				htod8a(&dpv[0],len+1);
 				for (i = 0; i < len; i++)
 					htod16a(&dpv[i*2+1],((uint16_t *)value)[i]);
-				((uint16_t *)dpv)[len]=0;
-				
+				/* terminator is done by memset above */
 				/*for (i = 0; i < size; i++)
 					printf("dpv[%d] = %d, ", i, dpv[i]);
 				printf("\n");*/
