@@ -1385,6 +1385,23 @@ ptp_free_devicepropdesc(PTPDevicePropDesc* dpd)
 	}
 }
 
+/* added by RAL 2006-01-01 */
+
+void
+ptp_free_deviceinfo(PTPDeviceInfo* di)
+{
+	free(di->VendorExtensionDesc);
+	free(di->OperationsSupported);
+	free(di->EventsSupported);
+	free(di->DevicePropertiesSupported);
+	free(di->CaptureFormats);
+	free(di->ImageFormats);
+	free(di->Manufacturer);
+  free(di->Model);
+  free(di->DeviceVersion);
+	free(di->SerialNumber);
+}
+
 /* report PTP errors */
 
 void 
